var bloosql = require('bloosql');
var randomstring = require('randomstring');
var config;

exports.configBasket = function (basketTable, basketItemsTable, basketIdentifier, basketRef, itemsTable, db, dbUser, dbPassword, callback){
    config = {
        basket : basketTable, /*for example 'blooware.basket'*/
        basketIdRow : basketIdentifier, /*for example 'basket'*/
        basketRef : basketRef, /*for example 'basket'*/
        basket_items : basketItemsTable,/*for example 'blooware.basket_items'*/
        itemsTable : itemsTable/*for example 'blooware.basket_items'*/
    }
    bloosql.setup(db,dbUser,dbPassword,3306,function(){callback(true)});

} /*setup basket*/

exports.createBasket = function (data, callback){
    var ref = randomstring.generate(30);
    bloosql.query("insert into ?? set ?, ref = ?",[config.basket, data, ref],function(rows, err, q){
        if(err){
            console.log("Error in Bloobasket : %s ", err);
            callback(false, ref);
        }
        else{            
            callback(true, ref);
        }
    });
} /*create a new basket*/

exports.getBasket = function (user, callback){
    bloosql.query("select * from ?? where user_id = ? order by id desc limit 1 ",[config.basket, user],function(rows, err, q){
        if(err){
            console.log("Error in Bloobasket : %s ", err);
            callback(rows[0]);
        }
        else{            
            callback(rows[0]);
        }
    });
}

exports.deleteBasket = function (id, callback){
    bloosql.query("delete from ?? where ?? = ?; ",[config.basket_items, config.basketIdRow, id],function(rows, err, q){
        if(err){
            console.log("Error in Bloobasket : %s ", err);

        }
        else{            
            deleteBask();
        }
    });

    function deleteBask(){
        bloosql.query(" delete from ?? where ?? = ?;",[config.basket, config.basketRef, id],function(rows, err, q){

            if(err){
                console.log("Error in Bloobasket : %s ", err);
                callback(false);
            }
            else{            
                callback(true);
            }
        });
    }
} /* delete a specific basket*/

exports.addToBasket = function (item, callback){
    bloosql.query("insert into ?? set ? ",[config.basket_items, item],function(rows, err, q){
        if(err){
            console.log("Error in Bloobasket : %s ", err);
            callback(false);
        }
        else{            
            callback(true);
        }
    });
} /*add item to basket*/

exports.getBasketItems = function (basketId, callback){
    bloosql.query("select * from ?? where ?? = ?",[config.basket_items, config.basketIdRow, basketId],function(rows, err, q){
        if(err){
            console.log("Error in Bloobasket : %s ", err);
        }
        else{            
            callback(rows);
        }
    });
} /*return all items in a basket*/

exports.removeFromBasket = function (item, callback){
    bloosql.query("delete from ?? where id = ? ",[config.basket_items, item],function(rows, err, q){
        if(err){
            console.log("Error in Bloobasket : %s ", err);
            callback(false);
        }
        else{            
            callback(true);
        }
    });
} /*remove item from basket*/

exports.checkOut = function (basketId, itemsTableId, basket_itemsId, callback, discount){
    var items;
    var cost;

    function getItems(callback){
        bloosql.query("select * from ?? inner join ?? on ??=?? where basket = ?",[config.basket_items, config.itemsTable, itemsTableId, basket_itemsId, basketId],function(rows, err, q){

            if(err){
                console.log("Error in Bloobasket : %s ", err);
            }
            else{  
                items = rows;
                callback();
            }
        }); 
    }

    function getCost(callback){
        bloosql.query("select sum(price) as price, sum(price - ((price / 100) * ??)) as discountPrice from ?? inner join pepper.items on ??=?? where basket = ?",["pepper.items.discount_id",config.basket_items, config.itemsTable, itemsTableId, basket_itemsId, basketId],function(rows, err, q){

            if(err){
                console.log("Error in Bloobasket : %s ", err);
            }
            else{
                cost = rows[0].price;
                discount = rows[0].discountPrice;
                callback();
            }
        });
    }
    
    function finish(){
        callback({cost : cost, items : items, discount : discount, discountCost : cost - discount});
    }
    
    getItems(function(){getCost(function(){finish()})});

}
